@@ -1,17 +1,27 @@
 extern crate pyroscope;
 
 use pyroscope::{PyroscopeAgent, Result};
+use pyroscope_pprofrs::{Pprof, PprofConfig};
+use std::hash::{Hash, Hasher};
 
-fn fibonacci(n: u64) -> u64 {
-    match n {
-        0 | 1 => 1,
-        n => fibonacci(n - 1) + fibonacci(n - 2),
+fn hash_rounds(n: u64) -> u64 {
+    let hash_str = "Some string to hash";
+    let mut default_hasher = std::collections::hash_map::DefaultHasher::new();
+
+    for _ in 0..n {
+        for _ in 0..1000 {
+            default_hasher.write(hash_str.as_bytes());
+        }
+        hash_str.hash(&mut default_hasher);
     }
+
+    n
 }
 
 fn main() -> Result<()> {
     let mut agent = PyroscopeAgent::builder("http://localhost:4040", "example.basic")
-        .tags(&[("TagA", "ValueA"), ("TagB", "ValueB")])
+        .backend(Pprof::new(PprofConfig::new().sample_rate(100)))
+        .tags([("TagA", "ValueA"), ("TagB", "ValueB")].to_vec())
         .build()?;
 
     // Show start time
@@ -22,9 +32,9 @@ fn main() -> Result<()> {
     println!("Start Time: {}", start);
 
     // Start Agent
-    agent.start();
+    agent.start()?;
 
-    let _result = fibonacci(47);
+    let _result = hash_rounds(300_000);
 
     // Show stop time
     let stop = std::time::SystemTime::now()
@@ -34,7 +44,7 @@ fn main() -> Result<()> {
     println!("Stop Time: {}", stop);
 
     // Stop Agent
-    agent.stop();
+    agent.stop()?;
 
     drop(agent);
 
