@@ -1,6 +1,7 @@
 extern crate pyroscope;
 
 use pyroscope::{PyroscopeAgent, Result};
+use pyroscope_pprofrs::{Pprof, PprofConfig};
 
 fn fibonacci(n: u64) -> u64 {
     match n {
@@ -17,15 +18,16 @@ fn main() -> Result<()> {
     pretty_env_logger::init_timed();
 
     let mut agent = PyroscopeAgent::builder("http://invalid_url", "example.error")
+        .backend(Pprof::new(PprofConfig::new().sample_rate(100)))
         .build()
         .unwrap();
     // Start Agent
-    agent.start();
+    agent.start()?;
 
     let _result = fibonacci(47);
 
     // Stop Agent
-    agent.stop();
+    agent.stop()?;
 
     drop(agent);
 
