@@ -1,19 +1,23 @@
 use std::{
     collections::HashMap,
     sync::{
-        mpsc::{channel, Receiver, Sender},
+        mpsc::{self, Sender},
         Arc, Condvar, Mutex,
     },
     thread::JoinHandle,
 };
 
 use crate::{
-    backends::{pprof::Pprof, Backend},
     error::Result,
     session::{Session, SessionManager, SessionSignal},
-    timer::Timer,
+    timer::{Timer, TimerSignal},
+    utils::get_time_range,
 };
 
+use crate::backend::{Backend, VoidBackend};
+
+const LOG_TAG: &str = "Pyroscope::Agent";
+
 /// Pyroscope Agent Configuration. This is the configuration that is passed to the agent.
 /// # Example
 /// ```
@@ -28,13 +32,10 @@ pub struct PyroscopeConfig {
     pub application_name: String,
     /// Tags
     pub tags: HashMap<String, String>,
-    /// Sample rate used in Hz
-    pub sample_rate: i32,
-    // TODO
-    // log_level
-    // auth_token
-    // upstream_request_timeout = 10s
-    // no_logging
+    /// Sample Rate
+    pub sample_rate: u32,
+    /// Spy Name
+    pub spy_name: String,
 }
 
 impl PyroscopeConfig {
@@ -44,38 +45,37 @@ impl PyroscopeConfig {
     /// ```ignore
     /// let config = PyroscopeConfig::new("http://localhost:8080", "my-app");
     /// ```
-    pub fn new<S: AsRef<str>>(url: S, application_name: S) -> Self {
+    pub fn new(url: impl AsRef<str>, application_name: impl AsRef<str>) -> Self {
         Self {
-            url: url.as_ref().to_owned(),
-            application_name: application_name.as_ref().to_owned(),
-            tags: HashMap::new(),
-            sample_rate: 100i32,
+            url: url.as_ref().to_owned(), // Pyroscope Server URL
+            application_name: application_name.as_ref().to_owned(), // Application Name
+            tags: HashMap::new(),         // Empty tags
+            sample_rate: 100u32,          // Default sample rate
+            spy_name: String::from("undefined"), // Spy Name should be set by the backend
         }
     }
 
-    /// Set the Sample rate
-    /// # Example
-    /// ```ignore
-    /// let mut config = PyroscopeConfig::new("http://localhost:8080", "my-app");
-    /// config.set_sample_rate(10)
-    /// .unwrap();
-    /// ```
-    pub fn sample_rate(self, sample_rate: i32) -> Self {
+    /// Set the Sample rate.
+    pub fn sample_rate(self, sample_rate: u32) -> Self {
         Self {
             sample_rate,
             ..self
         }
     }
 
+    /// Set the Spy Name.
+    pub fn spy_name(self, spy_name: String) -> Self {
+        Self { spy_name, ..self }
+    }
+
     /// Set the tags
     /// # Example
     /// ```ignore
     /// use pyroscope::pyroscope::PyroscopeConfig;
     /// let config = PyroscopeConfig::new("http://localhost:8080", "my-app")
-    ///    .tags(vec![("env", "dev")])
-    ///    .unwrap();
+    ///    .tags(vec![("env", "dev")])?;
     /// ```
-    pub fn tags(self, tags: &[(&str, &str)]) -> Self {
+    pub fn tags(self, tags: Vec<(&str, &str)>) -> Self {
         // Convert &[(&str, &str)] to HashMap(String, String)
         let tags_hashmap: HashMap<String, String> = tags
             .to_owned()
@@ -100,7 +100,7 @@ impl PyroscopeConfig {
 /// ```ignore
 /// use pyroscope::pyroscope::PyroscopeAgentBuilder;
 /// let builder = PyroscopeAgentBuilder::new("http://localhost:8080", "my-app");
-/// let agent = builder.build().unwrap();
+/// let agent = builder.build()?;
 /// ```
 pub struct PyroscopeAgentBuilder {
     /// Profiler backend
@@ -117,9 +117,9 @@ impl PyroscopeAgentBuilder {
     /// ```ignore
     /// let builder = PyroscopeAgentBuilder::new("http://localhost:8080", "my-app");
     /// ```
-    pub fn new<S: AsRef<str>>(url: S, application_name: S) -> Self {
+    pub fn new(url: impl AsRef<str>, application_name: impl AsRef<str>) -> Self {
         Self {
-            backend: Arc::new(Mutex::new(Pprof::default())), // Default Backend
+            backend: Arc::new(Mutex::new(VoidBackend::default())), // Default Backend
             config: PyroscopeConfig::new(url, application_name),
         }
     }
@@ -130,11 +130,11 @@ impl PyroscopeAgentBuilder {
     /// let builder = PyroscopeAgentBuilder::new("http://localhost:8080", "my-app")
     /// .backend(Pprof::default())
     /// .build()
-    /// .unwrap();
+    /// ?;
     /// ```
-    pub fn backend<T: 'static>(self, backend: T) -> Self
+    pub fn backend<T>(self, backend: T) -> Self
     where
-        T: Backend,
+        T: 'static + Backend,
     {
         Self {
             backend: Arc::new(Mutex::new(backend)),
@@ -142,30 +142,14 @@ impl PyroscopeAgentBuilder {
         }
     }
 
-    /// Set the Sample rate. Default value is 100.
-    /// # Example
-    /// ```ignore
-    /// let builder = PyroscopeAgentBuilder::new("http://localhost:8080", "my-app")
-    /// .sample_rate(99)
-    /// .build()
-    /// .unwrap();
-    /// ```
-    pub fn sample_rate(self, sample_rate: i32) -> Self {
-        Self {
-            config: self.config.sample_rate(sample_rate),
-            ..self
-        }
-    }
-
     /// Set tags. Default is empty.
     /// # Example
     /// ```ignore
     /// let builder = PyroscopeAgentBuilder::new("http://localhost:8080", "my-app")
     /// .tags(vec![("env", "dev")])
-    /// .build()
-    /// .unwrap();
+    /// .build()?;
     /// ```
-    pub fn tags(self, tags: &[(&str, &str)]) -> Self {
+    pub fn tags(self, tags: Vec<(&str, &str)>) -> Self {
         Self {
             config: self.config.tags(tags),
             ..self
@@ -174,23 +158,29 @@ impl PyroscopeAgentBuilder {
 
     /// Initialize the backend, timer and return a PyroscopeAgent object.
     pub fn build(self) -> Result<PyroscopeAgent> {
-        // Initiliaze the backend
+        // Get the backend
         let backend = Arc::clone(&self.backend);
-        backend.lock()?.initialize(self.config.sample_rate)?;
-        log::trace!("PyroscopeAgent - Backend initialized");
+        // Initialize the backend
+        backend.lock()?.initialize()?;
+
+        // Set Spy Name and Sample Rate from the Backend
+        let config = self.config.sample_rate(backend.lock()?.sample_rate()?);
+        let config = config.spy_name(backend.lock()?.spy_name()?);
+
+        log::trace!(target: LOG_TAG, "Backend initialized");
 
         // Start Timer
-        let timer = Timer::default().initialize()?;
-        log::trace!("PyroscopeAgent - Timer initialized");
+        let timer = Timer::initialize(std::time::Duration::from_secs(10))?;
+        log::trace!(target: LOG_TAG, "Timer initialized");
 
         // Start the SessionManager
         let session_manager = SessionManager::new()?;
-        log::trace!("PyroscopeAgent - SessionManager initialized");
+        log::trace!(target: LOG_TAG, "SessionManager initialized");
 
         // Return PyroscopeAgent
         Ok(PyroscopeAgent {
             backend: self.backend,
-            config: self.config,
+            config,
             timer,
             session_manager,
             tx: None,
@@ -205,7 +195,7 @@ impl PyroscopeAgentBuilder {
 pub struct PyroscopeAgent {
     timer: Timer,
     session_manager: SessionManager,
-    tx: Option<Sender<u64>>,
+    tx: Option<Sender<TimerSignal>>,
     handle: Option<JoinHandle<Result<()>>>,
     running: Arc<(Mutex<bool>, Condvar)>,
 
@@ -219,44 +209,47 @@ pub struct PyroscopeAgent {
 impl Drop for PyroscopeAgent {
     /// Properly shutdown the agent.
     fn drop(&mut self) {
-        log::debug!("PyroscopeAgent - Dropping Agent");
+        log::debug!(target: LOG_TAG, "PyroscopeAgent::drop()");
 
         // Drop Timer listeners
         match self.timer.drop_listeners() {
-            Ok(_) => log::trace!("PyroscopeAgent - Dropped timer listeners"),
-            Err(_) => log::error!("PyroscopeAgent - Error Dropping timer listeners"),
+            Ok(_) => log::trace!(target: LOG_TAG, "Dropped timer listeners"),
+            Err(_) => log::error!(target: LOG_TAG, "Error Dropping timer listeners"),
         }
 
         // Wait for the Timer thread to finish
         if let Some(handle) = self.timer.handle.take() {
             match handle.join() {
-                Ok(_) => log::trace!("PyroscopeAgent - Dropped timer thread"),
-                Err(_) => log::error!("PyroscopeAgent - Error Dropping timer thread"),
+                Ok(_) => log::trace!(target: LOG_TAG, "Dropped timer thread"),
+                Err(_) => log::error!(target: LOG_TAG, "Error Dropping timer thread"),
             }
         }
 
         // Stop the SessionManager
         match self.session_manager.push(SessionSignal::Kill) {
-            Ok(_) => log::trace!("PyroscopeAgent - Sent kill signal to SessionManager"),
-            Err(_) => log::error!("PyroscopeAgent - Error sending kill signal to SessionManager"),
+            Ok(_) => log::trace!(target: LOG_TAG, "Sent kill signal to SessionManager"),
+            Err(_) => log::error!(
+                target: LOG_TAG,
+                "Error sending kill signal to SessionManager"
+            ),
         }
 
         if let Some(handle) = self.session_manager.handle.take() {
             match handle.join() {
-                Ok(_) => log::trace!("PyroscopeAgent - Dropped SessionManager thread"),
-                Err(_) => log::error!("PyroscopeAgent - Error Dropping SessionManager thread"),
+                Ok(_) => log::trace!(target: LOG_TAG, "Dropped SessionManager thread"),
+                Err(_) => log::error!(target: LOG_TAG, "Error Dropping SessionManager thread"),
             }
         }
 
         // Wait for main thread to finish
         if let Some(handle) = self.handle.take() {
             match handle.join() {
-                Ok(_) => log::trace!("PyroscopeAgent - Dropped main thread"),
-                Err(_) => log::error!("PyroscopeAgent - Error Dropping main thread"),
+                Ok(_) => log::trace!(target: LOG_TAG, "Dropped main thread"),
+                Err(_) => log::error!(target: LOG_TAG, "Error Dropping main thread"),
             }
         }
 
-        log::debug!("PyroscopeAgent - Agent Dropped");
+        log::debug!(target: LOG_TAG, "Agent Dropped");
     }
 }
 
@@ -265,15 +258,21 @@ impl PyroscopeAgent {
     ///
     /// # Example
     /// ```ignore
-    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build().unwrap();
+    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build()?;
     /// ```
     pub fn builder<S: AsRef<str>>(url: S, application_name: S) -> PyroscopeAgentBuilder {
         // Build PyroscopeAgent
         PyroscopeAgentBuilder::new(url, application_name)
     }
 
-    fn _start(&mut self) -> Result<()> {
-        log::debug!("PyroscopeAgent - Starting");
+    /// Start profiling and sending data. The agent will keep running until stopped. The agent will send data to the server every 10s secondy.
+    /// # Example
+    /// ```ignore
+    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build()?;
+    /// agent.start()?;
+    /// ```
+    pub fn start(&mut self) -> Result<()> {
+        log::debug!(target: LOG_TAG, "Starting");
 
         // Create a clone of Backend
         let backend = Arc::clone(&self.backend);
@@ -287,7 +286,7 @@ impl PyroscopeAgent {
         *running = true;
         drop(running);
 
-        let (tx, rx): (Sender<u64>, Receiver<u64>) = channel();
+        let (tx, rx) = mpsc::channel();
         self.timer.attach_listener(tx.clone())?;
         self.tx = Some(tx);
 
@@ -296,30 +295,33 @@ impl PyroscopeAgent {
         let stx = self.session_manager.tx.clone();
 
         self.handle = Some(std::thread::spawn(move || {
-            log::trace!("PyroscopeAgent - Main Thread started");
+            log::trace!(target: LOG_TAG, "Main Thread started");
 
-            while let Ok(until) = rx.recv() {
-                log::trace!("PyroscopeAgent - Sending session {}", until);
+            while let Ok(signal) = rx.recv() {
+                match signal {
+                    TimerSignal::NextSnapshot(until) => {
+                        log::trace!(target: LOG_TAG, "Sending session {}", until);
 
-                // Generate report from backend
-                let report = backend.lock()?.report()?;
+                        // Generate report from backend
+                        let report = backend.lock()?.report()?;
 
-                // Send new Session to SessionManager
-                stx.send(SessionSignal::Session(Session::new(
-                    until,
-                    config.clone(),
-                    report,
-                )?))?;
+                        // Send new Session to SessionManager
+                        stx.send(SessionSignal::Session(Session::new(
+                            until,
+                            config.clone(),
+                            report,
+                        )?))?
+                    }
+                    TimerSignal::Terminate => {
+                        log::trace!(target: LOG_TAG, "Session Killed");
 
-                if until == 0 {
-                    log::trace!("PyroscopeAgent - Session Killed");
+                        let (lock, cvar) = &*pair;
+                        let mut running = lock.lock()?;
+                        *running = false;
+                        cvar.notify_one();
 
-                    let (lock, cvar) = &*pair;
-                    let mut running = lock.lock()?;
-                    *running = false;
-                    cvar.notify_one();
-
-                    return Ok(());
+                        return Ok(());
+                    }
                 }
             }
             Ok(())
@@ -328,23 +330,24 @@ impl PyroscopeAgent {
         Ok(())
     }
 
-    /// Start profiling and sending data. The agent will keep running until stopped. The agent will send data to the server every 10s secondy.
+    /// Stop the agent. The agent will stop profiling and send a last report to the server.
     /// # Example
     /// ```ignore
-    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build().unwrap();
-    /// agent.start();
+    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build()?;
+    /// agent.start()?;
+    /// // Expensive operation
+    /// agent.stop();
     /// ```
-    pub fn start(&mut self) {
-        match self._start() {
-            Ok(_) => log::trace!("PyroscopeAgent - Agent started"),
-            Err(_) => log::error!("PyroscopeAgent - Error starting agent"),
-        }
-    }
-
-    fn _stop(&mut self) -> Result<()> {
-        log::debug!("PyroscopeAgent - Stopping");
+    pub fn stop(&mut self) -> Result<()> {
+        log::debug!(target: LOG_TAG, "Stopping");
         // get tx and send termination signal
-        self.tx.take().unwrap().send(0)?;
+        if let Some(sender) = self.tx.take() {
+            // Send last session
+            let _ = sender.send(TimerSignal::NextSnapshot(get_time_range(0)?.until));
+            sender.send(TimerSignal::Terminate)?;
+        } else {
+            log::error!("PyroscopeAgent - Missing sender")
+        }
 
         // Wait for the Thread to finish
         let pair = Arc::clone(&self.running);
@@ -359,40 +362,25 @@ impl PyroscopeAgent {
         Ok(())
     }
 
-    /// Stop the agent. The agent will stop profiling and send a last report to the server.
-    /// # Example
-    /// ```ignore
-    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build().unwrap();
-    /// agent.start();
-    /// // Expensive operation
-    /// agent.stop();
-    /// ```   
-    pub fn stop(&mut self) {
-        match self._stop() {
-            Ok(_) => log::trace!("PyroscopeAgent - Agent stopped"),
-            Err(_) => log::error!("PyroscopeAgent - Error stopping agent"),
-        }
-    }
-
     /// Add tags. This will restart the agent.
     /// # Example
     /// ```ignore
-    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build().unwrap();
-    /// agent.start();
+    /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app").build()?;
+    /// agent.start()?;
     /// // Expensive operation
-    /// agent.add_tags(vec![("tag", "value")]).unwrap();
+    /// agent.add_tags(vec!["tag", "value"])?;
     /// // Tagged operation
-    /// agent.stop();
+    /// agent.stop()?;
     /// ```
     pub fn add_tags(&mut self, tags: &[(&str, &str)]) -> Result<()> {
-        log::debug!("PyroscopeAgent - Adding tags");
+        log::debug!(target: LOG_TAG, "Adding tags");
         // Check that tags are not empty
         if tags.is_empty() {
             return Ok(());
         }
 
         // Stop Agent
-        self.stop();
+        self.stop()?;
 
         // Convert &[(&str, &str)] to HashMap(String, String)
         let tags_hashmap: HashMap<String, String> = tags
@@ -405,7 +393,7 @@ impl PyroscopeAgent {
         self.config.tags.extend(tags_hashmap);
 
         // Restart Agent
-        self.start();
+        self.start()?;
 
         Ok(())
     }
@@ -413,16 +401,22 @@ impl PyroscopeAgent {
     /// Remove tags. This will restart the agent.
     /// # Example
     /// ```ignore
+    /// # use pyroscope::*;
+    /// # use std::result;
+    /// # fn main() -> result::Result<(), error::PyroscopeError> {
     /// let agent = PyroscopeAgent::builder("http://localhost:8080", "my-app")
     /// .tags(vec![("tag", "value")])
-    /// .build().unwrap();
-    /// agent.start();
+    /// .build()?;
+    /// agent.start()?;
     /// // Expensive operation
-    /// agent.remove_tags(vec!["tag"]).unwrap();
+    /// agent.remove_tags(vec!["tag"])?;
     /// // Un-Tagged operation
-    /// agent.stop();
+    /// agent.stop()?;
+    /// # Ok(())
+    /// # }
+    /// ```
     pub fn remove_tags(&mut self, tags: &[&str]) -> Result<()> {
-        log::debug!("PyroscopeAgent - Removing tags");
+        log::debug!(target: LOG_TAG, "Removing tags");
 
         // Check that tags are not empty
         if tags.is_empty() {
@@ -430,7 +424,7 @@ impl PyroscopeAgent {
         }
 
         // Stop Agent
-        self.stop();
+        self.stop()?;
 
         // Iterate through every tag
         tags.iter().for_each(|key| {
@@ -439,7 +433,7 @@ impl PyroscopeAgent {
         });
 
         // Restart Agent
-        self.start();
+        self.start()?;
 
         Ok(())
     }
