@@ -1,13 +1,16 @@
 use std::{
     sync::mpsc::{sync_channel, Receiver, SyncSender},
-    thread,
-    thread::JoinHandle,
+    thread::{self, JoinHandle},
+    time::Duration,
 };
 
-use crate::pyroscope::PyroscopeConfig;
-use crate::utils::get_time_range;
-use crate::utils::merge_tags_with_app_name;
-use crate::Result;
+use crate::{
+    pyroscope::PyroscopeConfig,
+    utils::{get_time_range, merge_tags_with_app_name},
+    Result,
+};
+
+const LOG_TAG: &str = "Pyroscope::Session";
 
 /// Session Signal
 ///
@@ -32,15 +35,14 @@ pub struct SessionManager {
 impl SessionManager {
     /// Create a new SessionManager
     pub fn new() -> Result<Self> {
-        log::info!("SessionManager - Creating SessionManager");
+        log::info!(target: LOG_TAG, "Creating SessionManager");
 
         // Create a channel for sending and receiving sessions
         let (tx, rx): (SyncSender<SessionSignal>, Receiver<SessionSignal>) = sync_channel(10);
 
         // Create a thread for the SessionManager
         let handle = Some(thread::spawn(move || {
-            log::trace!("SessionManager - SessionManager thread started");
-            // This thread should only return if a kill signal is received.
+            log::trace!(target: LOG_TAG, "Started");
             while let Ok(signal) = rx.recv() {
                 match signal {
                     SessionSignal::Session(session) => {
@@ -54,7 +56,7 @@ impl SessionManager {
                     }
                     SessionSignal::Kill => {
                         // Kill the session manager
-                        log::trace!("SessionManager - Kill signal received");
+                        log::trace!(target: LOG_TAG, "Kill signal received");
                         return Ok(());
                     }
                 }
@@ -70,7 +72,7 @@ impl SessionManager {
         // Push the session into the SessionManager
         self.tx.send(session)?;
 
-        log::trace!("SessionManager - SessionSignal pushed");
+        log::trace!(target: LOG_TAG, "SessionSignal pushed");
 
         Ok(())
     }
@@ -83,7 +85,9 @@ impl SessionManager {
 pub struct Session {
     pub config: PyroscopeConfig,
     pub report: Vec<u8>,
+    // unix time
     pub from: u64,
+    // unix time
     pub until: u64,
 }
 
@@ -94,10 +98,10 @@ impl Session {
     /// let config = PyroscopeConfig::new("https://localhost:8080", "my-app");
     /// let report = vec![1, 2, 3];
     /// let until = 154065120;
-    /// let session = Session::new(until, config, report).unwrap();
+    /// let session = Session::new(until, config, report)?;
     /// ```
     pub fn new(until: u64, config: PyroscopeConfig, report: Vec<u8>) -> Result<Self> {
-        log::info!("Session - Creating Session");
+        log::info!(target: LOG_TAG, "Creating Session");
 
         // get_time_range should be used with "from". We balance this by reducing
         // 10s from the returned range.
@@ -117,11 +121,16 @@ impl Session {
     /// let config = PyroscopeConfig::new("https://localhost:8080", "my-app");
     /// let report = vec![1, 2, 3];
     /// let until = 154065120;
-    /// let session = Session::new(until, config, report).unwrap();
-    /// session.send().unwrap();
+    /// let session = Session::new(until, config, report)?;
+    /// session.send()?;
     /// ```
     pub fn send(self) -> Result<()> {
-        log::info!("Session - Sending Session {} - {}", self.from, self.until);
+        log::info!(
+            target: LOG_TAG,
+            "Sending Session: {} - {}",
+            self.from,
+            self.until
+        );
 
         // Check if the report is empty
         if self.report.is_empty() {
@@ -150,10 +159,10 @@ impl Session {
                 ("until", &format!("{}", self.until)),
                 ("format", "folded"),
                 ("sampleRate", &format!("{}", self.config.sample_rate)),
-                ("spyName", "pyroscope-rs"),
+                ("spyName", self.config.spy_name.as_str()),
             ])
             .body(self.report)
-            .timeout(std::time::Duration::from_secs(10))
+            .timeout(Duration::from_secs(10))
             .send()?;
 
         Ok(())
