@@ -1,14 +1,8 @@
 use pyroscope::{
     pyroscope::PyroscopeConfig,
     session::{Session, SessionManager, SessionSignal},
-    PyroscopeError,
-};
-use std::{
-    collections::HashMap,
-    sync::mpsc::{sync_channel, Receiver, SyncSender},
-    thread,
-    thread::JoinHandle,
 };
+use std::collections::HashMap;
 
 #[test]
 fn test_session_manager_new() {
@@ -29,7 +23,8 @@ fn test_session_new() {
         url: "http://localhost:8080".to_string(),
         application_name: "test".to_string(),
         tags: HashMap::new(),
-        sample_rate: 100,
+        sample_rate: 100u32,
+        spy_name: "test-rs".to_string(),
     };
 
     let report = vec![1, 2, 3];
@@ -46,12 +41,13 @@ fn test_session_send_error() {
         url: "http://invalid_url".to_string(),
         application_name: "test".to_string(),
         tags: HashMap::new(),
-        sample_rate: 100,
+        sample_rate: 100u32,
+        spy_name: "test-rs".to_string(),
     };
 
     let report = vec![1, 2, 3];
 
-    let session = Session::new(1950, config, report).unwrap();
+    let _session = Session::new(1950, config, report).unwrap();
 
     // TODO: to figure this out
 }
